#include "dateinfowidget.h"

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QGridLayout>

DateInfoWidget::DateInfoWidget(QWidget *parent)
    : QWidget(parent)
{
    // Create the various widgets
    m_dateSelectWidget = new DateSelectionWidget(this);

    m_dateFormatComboBox = new QComboBox(this);
    m_dateFormatStringEdit = new QLineEdit(this);

    m_dateStringLabel = new QLabel(this);
    m_weekNumberLabel = new QLabel(this);
    m_dayOfWeekLabel = new QLabel(this);
    m_dayOfYearLabel = new QLabel(this);
    m_julianDayLabel = new QLabel(this);
    m_isLeapYearLabel = new QLabel(this);

    m_deltaEdit = new QLineEdit(this);
    m_intervalTypeComboBox = new QComboBox(this);
    m_beforeAfterComboBox = new QComboBox(this);
    m_updatedDateLabel = new QLabel(this);

    // Create layout containing widgets
    QVBoxLayout *vLayout = new QVBoxLayout(this);
    vLayout->addWidget(m_dateSelectWidget);

    QHBoxLayout *formatLayout = new QHBoxLayout();
    formatLayout->addWidget(new QLabel("Date Format: ", this));
    formatLayout->addWidget(m_dateFormatComboBox);
    formatLayout->addWidget(m_dateFormatStringEdit);

    QGridLayout *infoLayout = new QGridLayout();
    infoLayout->addWidget(m_dateStringLabel, 0, 0);
    infoLayout->addWidget(new QLabel("Week #", this), 1, 0);
    infoLayout->addWidget(m_weekNumberLabel, 1, 1);
    infoLayout->addWidget(new QLabel("Day of year: ", this), 1, 2);
    infoLayout->addWidget(m_dayOfYearLabel, 1, 3);
    infoLayout->addWidget(new QLabel("Day of week: ", this), 2, 0);
    infoLayout->addWidget(m_dayOfWeekLabel, 2, 1);
    infoLayout->addWidget(new QLabel("Leap Year: ", this), 2, 2);
    infoLayout->addWidget(m_isLeapYearLabel, 2, 3);
    infoLayout->addWidget(new QLabel("Julian day: ", this), 3, 0);
    infoLayout->addWidget(m_julianDayLabel, 3, 1);

    QHBoxLayout *updateDateLayout = new QHBoxLayout();
    updateDateLayout->addWidget(new QLabel("Date ", this), 1);
    updateDateLayout->addWidget(m_deltaEdit, 1);
    updateDateLayout->addWidget(m_intervalTypeComboBox, 1);
    updateDateLayout->addWidget(m_beforeAfterComboBox, 1);
    updateDateLayout->addWidget(m_updatedDateLabel, 5);

    vLayout->addLayout(formatLayout);
    vLayout->addLayout(infoLayout);
    vLayout->addLayout(updateDateLayout);

    // String representation of days of the week.
    m_dayList << "Monday" << "Tuesday" << "Wednesday" << "Thursday" << "Friday" << "Saturday" << "Sunday";

    // Set up and then update the UI for the first time
    setupUI();
    updateUI();
}

DateInfoWidget::~DateInfoWidget()
{

}

void DateInfoWidget::onDateChanged(QDate newDate)
{
    Q_UNUSED(newDate);
    updateUI();
}

void DateInfoWidget::onDateFormatChanged(int format)
{
    m_dateFormatStringEdit->setEnabled(!format);
    updateUI();
}

void DateInfoWidget::onDateFormatStringChanged(const QString& format)
{
    Q_UNUSED(format);
    updateUI();
}

void DateInfoWidget::onDeltaChanged(const QString& delta)
{
    Q_UNUSED(delta);
    updateUI();
}

void DateInfoWidget::onIntervalTypeChanged(int type)
{
    Q_UNUSED(type);
    updateUI();
}

void DateInfoWidget::setupUI()
{
    // Set up the combo box contents
    QStringList dateFormatNameList;
    dateFormatNameList << "Use Format String:"
                       << "Text Date" << "ISO Date" << "ISO Date With ms"
                       << "System Locale Short Date" << "System Locale Long Date"
                       << "Default Locale Short Date" << "Default Locale Long Date"
                       << "System Locale Date" << "Locale Date" << "Local Date" // deprecated but included for completeness
                       << "RFC 2822 Date";

    QList<Qt::DateFormat> dateFormatList;
    dateFormatList   << Qt::TextDate << Qt::ISODate << Qt::ISODateWithMs
                     << Qt::SystemLocaleShortDate << Qt::SystemLocaleLongDate
                     << Qt::DefaultLocaleShortDate << Qt::SystemLocaleLongDate
                     << Qt::SystemLocaleDate << Qt::LocalDate << Qt::LocalDate // deprecated but included for completeness
                     << Qt::RFC2822Date;

    m_dateFormatComboBox->addItems(dateFormatNameList);
    for (int i = 1; i < dateFormatNameList.count(); i++)
    {
        m_dateFormatComboBox->setItemData(i, dateFormatList[i-1]);
    }
    m_dateFormatComboBox->setCurrentIndex(1);
    m_dateFormatStringEdit->setText("MM/dd/yyyy");
    m_dateFormatStringEdit->setEnabled(false);

    m_intervalTypeComboBox->addItem("day(s)");
    m_intervalTypeComboBox->addItem("week(s)");
    m_intervalTypeComboBox->addItem("month(s)");
    m_intervalTypeComboBox->addItem("year(s)");

    m_beforeAfterComboBox->addItem("before");
    m_beforeAfterComboBox->addItem("after");

    // Set up validator for time delta editor -- only allow integers
    QValidator *validator = new QIntValidator(this);
    m_deltaEdit->setValidator(validator);
    m_deltaEdit->setText("0");

    // Set up connections for UI elements
    connect(m_dateFormatComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &DateInfoWidget::onDateFormatChanged);
    connect(m_dateSelectWidget, &DateSelectionWidget::dateChanged, this, &DateInfoWidget::onDateChanged);
    connect(m_dateFormatStringEdit, &QLineEdit::textChanged, this, &DateInfoWidget::onDateFormatStringChanged);
    connect(m_deltaEdit, &QLineEdit::textChanged, this, &DateInfoWidget::onDeltaChanged);
    connect(m_intervalTypeComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &DateInfoWidget::onIntervalTypeChanged);
    connect(m_beforeAfterComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &DateInfoWidget::onIntervalTypeChanged);
}

// Update the UI after the user changes the date or some other setting
void DateInfoWidget::updateUI()
{
    QDate date = m_dateSelectWidget->getDate();

    // If invalid date then update the UI to reflect that
    if (!date.isValid())
    {
        updateUIInvalidDate();
        return;
    }

    // Get information about specified date
    int yearNumber;
    int weekNumber = date.weekNumber(&yearNumber);
    QString dayOfWeek = getDayOfWeek(date.dayOfWeek());
    int dayOfYear = date.dayOfYear();
    qint64 julianDay = date.toJulianDay();
    QString isLeapYear = QDate::isLeapYear(date.year()) ? "true" : "false";

    m_weekNumberLabel->setText(QString("%1 of %2").arg(weekNumber).arg(yearNumber));
    m_dayOfWeekLabel->setText(dayOfWeek);
    m_dayOfYearLabel->setText(QString("%1").arg(dayOfYear));
    m_julianDayLabel->setText(QString("%1").arg(julianDay));
    m_isLeapYearLabel->setText(isLeapYear);

    QString dateString = getDateString(date);
    m_dateStringLabel->setText(dateString);

    QString updatedDateString = getUpdatedDateString(date);
    m_updatedDateLabel->setText(updatedDateString);
}

// If the selected date is invalid (which is only possible in
// this example if the year is 0) then update the UI
// to reflect this
void DateInfoWidget::updateUIInvalidDate()
{
    m_weekNumberLabel->setText(QString("N/A"));
    m_dayOfWeekLabel->setText(QString("N/A"));
    m_dayOfYearLabel->setText(QString("N/A"));
    m_julianDayLabel->setText(QString("N/A"));
    m_isLeapYearLabel->setText(QString("N/A"));
    m_dateStringLabel->setText(QString("Invalid Date"));
    m_updatedDateLabel->setText(QString("Invalid Date"));
}

// Get the string representation of the specified date based
// on the currently set format
QString DateInfoWidget::getDateString(QDate date) const
{
    int index = m_dateFormatComboBox->currentIndex();
    if (index == 0)
    {
        // Use string format specified in edit box
        return date.toString(m_dateFormatStringEdit->text());
    }
    else
    {
        // Use format specified in combo box
        int data = m_dateFormatComboBox->currentData().toInt();
        Qt::DateFormat format = static_cast<Qt::DateFormat>(data);

        return date.toString(format);
    }
}

// Get the string representation of the specififed date
// modified by the user-specified number of intervals
QString DateInfoWidget::getUpdatedDateString(QDate date) const
{
    int delta = m_deltaEdit->text().toInt();
    int intervalIndex = m_intervalTypeComboBox->currentIndex();
    int beforeIndex = m_beforeAfterComboBox->currentIndex();

    // If looking for a date before the current one, use a negative delta
    if (beforeIndex == 0) delta = -delta;

    // Get a new date
    QDate updatedDate;
    if (intervalIndex == 0) // days
        updatedDate = date.addDays(delta);
    else if (intervalIndex == 1) // weeks
        updatedDate = date.addDays(delta*7);
    else if (intervalIndex == 2) // months
        updatedDate = date.addMonths(delta);
    else // years
        updatedDate = date.addYears(delta);

    // Return string representation of new date
    return getDateString(updatedDate);
}

// Gets string representation of a day of the week
// 1 = Monday, 2 = Tuesday, etc.
// There is a function, QDate::longDayName that can do
// the same thing but it is now considered obsolete
QString DateInfoWidget::getDayOfWeek(int day) const
{
    if (day <= 0 || day > m_dayList.count()) return QString();

    return m_dayList[day - 1];
}
