#include "dateselectionwidget.h"

#include <QHBoxLayout>
#include <QLabel>

// DateSelectionWidget
// This widget allows a user to select a date by choosing the month and day
// from drop-down combo boxes, and a year by typing it in a line editor.
// The widget stores the date and can also emit a signal when the date
// is changed by the user.

// Constructor for DateSelectionWidget
DateSelectionWidget::DateSelectionWidget(QWidget *parent) : QWidget(parent)
{
    m_date = QDate::currentDate();

    m_monthComboBox = new QComboBox(this);
    m_dayComboBox = new QComboBox(this);
    m_yearLineEdit = new QLineEdit(this);

    QHBoxLayout *hLayout = new QHBoxLayout(this);
    hLayout->addWidget(new QLabel("Month"));
    hLayout->addWidget(m_monthComboBox);
    hLayout->addWidget(new QLabel("Day"));
    hLayout->addWidget(m_dayComboBox);
    hLayout->addWidget(new QLabel("Year"));
    hLayout->addWidget(m_yearLineEdit);

    setupUI();
}

// Set up UI on creating the widget
void DateSelectionWidget::setupUI()
{
    // Set up year edit field
    QValidator *validator = new QIntValidator(this);
    m_yearLineEdit->setValidator(validator);
    m_yearLineEdit->setText(QString("%1").arg(m_date.year()));

    // Set up month combo box
    QStringList monthList = { "January", "February", "March", "April", "May", "June",
                              "July", "August", "September", "October", "November", "December" };
    m_monthComboBox->addItems(monthList);
    m_monthComboBox->setCurrentIndex(m_date.month() - 1);

    // Set up day combo box
    updateDayComboBox(m_date.year(), m_date.month(), m_date.day());

    // Set up connections
    connect(m_monthComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &DateSelectionWidget::onMonthChanged);
    connect(m_dayComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &DateSelectionWidget::onDayChanged);
    connect(m_yearLineEdit, &QLineEdit::textChanged, this, &DateSelectionWidget::onYearChanged);
}

// When the month is changed, update the day combo box
// so it has the correct number of days.
void DateSelectionWidget::onMonthChanged(int month)
{
    int year = m_yearLineEdit->text().toInt();
    int day = m_dayComboBox->currentIndex() + 1;

    updateDayComboBox(year, month + 1, day);
}

// When the day is changed, update the internal date.
// Nothing else needs to be done as nothing else
// depends on the day value.
void DateSelectionWidget::onDayChanged(int day)
{
    // day may be less than zero when day combo box is cleared;
    // that sends a signal that the combo box's current index
    // is -1
    if (day < 0) return;

    int year = m_yearLineEdit->text().toInt();
    int month = m_monthComboBox->currentIndex() + 1;
    updateDate(year, month, day + 1);
}

// When the year is changed, update the day combo box, but
// only if the current month is February, as it's the only month
// in which the number of days could change if the year changes.
void DateSelectionWidget::onYearChanged(const QString& year)
{
    int month = m_monthComboBox->currentIndex() + 1;
    int day = m_dayComboBox->currentIndex() + 1;

    // Invalid date -- update the current date anyway, but don't
    // update the UI
    if (year.toInt() == 0)
        updateDate(year.toInt(), month, day);

    if (month == 2)
        updateDayComboBox(year.toInt(), month, day);
    else
        updateDate(year.toInt(), month, day);
}

// Update the day combo box, based on the specified month and year.
// We also need the day as we may need to update the current day
// if the month is changed to one with fewer days as the old month.
void DateSelectionWidget::updateDayComboBox(int year, int month, int day)
{
    // A temporary date used just to check how many days
    // are in the specified month.
    QDate placeholder;

    // If current year is invalid use year 1 as placeholder
    if (year == 0)
        placeholder = QDate(1, month, 1);
    else
        placeholder = QDate(year, month, 1);
    int daysInMonth = placeholder.daysInMonth();

    // Clear out the combo box and repopulate it.
    m_dayComboBox->clear();
    for (int i = 1; i < daysInMonth + 1; i++)
        m_dayComboBox->addItem(QString("%1").arg(i));

    if (day > daysInMonth)
    {
        // If the current day is greater than the number of days
        // in the month, set the current day to the last day of the month.
        m_dayComboBox->setCurrentIndex(daysInMonth - 1);
        updateDate(year, month, daysInMonth);
    }
    else
    {
        // If the current day already falls within the current month,
        // keep that the same.
        m_dayComboBox->setCurrentIndex(day - 1);
        updateDate(year, month, day);
    }
}

// Update the internal date and emit a signal that it has changed,
// assuming that it has changed.
void DateSelectionWidget::updateDate(int year, int month, int day)
{
    if (m_date.year() != year ||
        m_date.month() != month ||
        m_date.day() != day)
    {
        m_date.setDate(year, month, day);
        emit dateChanged(m_date);
    }
}

